/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music.components

import dev.schlaubi.lavakord.audio.*
import dev.schlaubi.lavakord.audio.player.Player
import dev.schlaubi.lavakord.rest.models.PartialTrack
import net.moonleay.botendo.util.Logger
import java.util.concurrent.BlockingQueue
import java.util.concurrent.LinkedBlockingQueue

class GuildTrackScheduler(val pl: Player) {

    private var queue: BlockingQueue<PartialTrack> = LinkedBlockingQueue()
    var repeating = false
    private var hasRegisteredEvents = false

    ///Add a track to queue and start playing, if there is no song currently playing
    suspend fun queue(track: PartialTrack, type: MusicManager.AddType) {
        if (this.pl.playingTrack == null) {
            play(track)
            return
        }
        when (type) {
            MusicManager.AddType.QUEUE -> {
                queue.offer(track)
            }

            MusicManager.AddType.UPSERT -> {
                val nq = LinkedBlockingQueue<PartialTrack>()
                nq.offer(track)
                for (t in queue.toList()) {
                    nq.offer(t)
                }
                queue = nq
            }
        }
    }

    suspend fun playNext(link: Link) {
        if (!queue.isEmpty())
            this.pl.playTrack(queue.poll())
        else {
            this.pl.stopTrack()
            link.destroy()
            clear()
        }
    }

    private suspend fun play(tr: PartialTrack) {
        this.pl.playTrack(tr)
    }

    fun addEvents() {
        if (hasRegisteredEvents)
            return
        hasRegisteredEvents = true

        Logger.out("Adding track events to GuildTrackScheduler...")

        pl.on<Event, TrackEndEvent> {
            onTrackEnd(this)
        }

        pl.on<Event, TrackStuckEvent> {
            onTrackStuck(this)
        }

        pl.on<Event, TrackExceptionEvent> {
            onTrackExc(this)
        }
    }

    private suspend fun onTrackEnd(e: TrackEndEvent) {
        if (e.reason.mayStartNext) {
            if (repeating) {
                Logger.out("Repeating track...")
                this.pl.playTrack(e.getTrack().copy())
                return
            }
            Logger.out("Track has ended; Playing next...")
            playNext(net.moonleay.botendo.Bot.lava.getLink(e.guildId))
        }
    }

    private suspend fun onTrackStuck(e: TrackStuckEvent) {
        Logger.out("Track is stuck, retrying...")
        this.pl.playTrack(e.getTrack().copy())
    }

    private suspend fun onTrackExc(e: TrackExceptionEvent) {
        Logger.out("Track had an exception, retrying...")
        this.pl.playTrack(e.getTrack().copy())
    }

    fun clear() {
        Logger.out("Clearing queue...")
        repeating = false
        queue.clear()
    }

    fun getQueue(): List<PartialTrack> {
        return queue.toList()
    }

    fun getHead(): PartialTrack {
        return queue.first()
    }

    fun isEmpty(): Boolean {
        return queue.isEmpty()
    }
}

